CREATE SEQUENCE acl_sid_id_seq START WITH 2000;
CREATE SEQUENCE acl_class_id_seq START WITH 2000;
CREATE SEQUENCE aoi_id_seq START WITH 2000;
CREATE SEQUENCE ae_id_seq START WITH 2000;


CREATE TABLE ACL_SID (
  ID		NUMBER (20)	NOT NULL,
  SID		VARCHAR2 (120)	NOT NULL,
  PRINCIPAL	NUMBER (8)	NOT NULL
);

 ALTER TABLE ACL_SID
  ADD  CONSTRAINT AS_PK PRIMARY KEY (ID);

 ALTER TABLE ACL_SID
  ADD  CONSTRAINT AS_UK UNIQUE (SID, PRINCIPAL);


CREATE OR REPLACE TRIGGER acl_sid_pk_trg  
BEFORE INSERT ON ACL_SID 
FOR EACH ROW
BEGIN     
IF :NEW.id IS NULL 
THEN      
SELECT acl_sid_id_seq.NEXTVAL INTO :NEW.id FROM DUAL;    
END IF;
END;

/
CREATE TABLE ACL_CLASS (
  ID	NUMBER (20)	NOT NULL,
  CLASS	VARCHAR2 (120) 	NOT NULL,
  CONSTRAINT AC_PK PRIMARY KEY (ID),
  CONSTRAINT AC_UK UNIQUE (CLASS)
);



CREATE OR REPLACE TRIGGER acl_class_pk_trg  
BEFORE INSERT ON ACL_CLASS 
FOR EACH ROW
BEGIN     
IF :NEW.id IS NULL 
THEN      
SELECT acl_class_id_seq.NEXTVAL INTO :NEW.id FROM DUAL;    
END IF;
END;
/
CREATE TABLE ACL_OBJECT_IDENTITY (
  ID			NUMBER (20)	NOT NULL,
  OBJECT_ID_CLASS	NUMBER (20)	NOT NULL,
  OBJECT_ID_IDENTITY	NUMBER (20)	NOT NULL,
  PARENT_OBJECT		NUMBER (20),
  OWNER_SID		NUMBER (20),
  ENTRIES_INHERITING    NUMBER (10)	NOT NULL,
  CONSTRAINT AOI_PK PRIMARY KEY (ID),
  CONSTRAINT AOI_UK UNIQUE (OBJECT_ID_CLASS, OBJECT_ID_IDENTITY)
);



ALTER TABLE ACL_OBJECT_IDENTITY
  ADD CONSTRAINT AOI_FK_AOI_PO
      FOREIGN KEY (PARENT_OBJECT)
      REFERENCES ACL_OBJECT_IDENTITY (ID);

ALTER TABLE ACL_OBJECT_IDENTITY
  ADD CONSTRAINT AOI_FK_AC_PO
      FOREIGN KEY (OBJECT_ID_CLASS)
      REFERENCES ACL_CLASS (ID);

ALTER TABLE ACL_OBJECT_IDENTITY
  ADD CONSTRAINT AOI_FK_AS_SID
      FOREIGN KEY (OWNER_SID)
      REFERENCES ACL_SID (ID);


CREATE OR REPLACE TRIGGER aoi_pk_trg  
BEFORE INSERT ON ACL_OBJECT_IDENTITY 
FOR EACH ROW
BEGIN     
IF :NEW.id IS NULL 
THEN      
SELECT aoi_id_seq.NEXTVAL INTO :NEW.id FROM DUAL;    
END IF;
END;

/
CREATE TABLE ACL_ENTRY (
  ID			NUMBER (20)	NOT NULL,
  ACL_OBJECT_IDENTITY	NUMBER (20)	NOT NULL,
  ACE_ORDER		NUMBER (10)	NOT NULL,
  SID			NUMBER (20)	NOT NULL,
  MASK      		NUMBER (10)	NOT NULL,
  GRANTING		NUMBER (1)	NOT NULL,
  AUDIT_SUCCESS		NUMBER (1)	NOT NULL,
  AUDIT_FAILURE		NUMBER (1)	NOT NULL,
  CONSTRAINT AE_PK PRIMARY KEY (ID),
  CONSTRAINT AE_UK UNIQUE ( ACL_OBJECT_IDENTITY, ACE_ORDER)
);


ALTER TABLE ACL_ENTRY
  ADD CONSTRAINT AE_FK_AOI_OII
      FOREIGN KEY (ACL_OBJECT_IDENTITY)
      REFERENCES ACL_OBJECT_IDENTITY (ID);

ALTER TABLE ACL_ENTRY
  ADD CONSTRAINT AE_FK_AS_SID
      FOREIGN KEY (SID)
      REFERENCES ACL_SID (ID);


CREATE OR REPLACE TRIGGER ae_pk_trg  
BEFORE INSERT ON ACL_ENTRY 
FOR EACH ROW
BEGIN     
IF :NEW.id IS NULL 
THEN      
SELECT ae_id_seq.NEXTVAL INTO :NEW.id FROM DUAL;    
END IF;
END;
/